/* eslint-disable @typescript-eslint/no-unsafe-call */
import { Module, MutationTree, ActionTree, GetterTree } from 'vuex';
import { api } from 'boot/axios';
import { AxiosError, AxiosResponse } from 'axios';
import { SessionStorage } from 'quasar';
import { CurrentUserResponse } from 'src/plugins/user/models';
import { UserSessionState } from '.';

export interface UserStateInterface {
  currentUser?: FG.User;
  currentPermissions: FG.Permission[];
  users: FG.User[];
  roles: FG.Role[];
  permissions: FG.Permission[];
  loading: number;
}

function loadUserFromLocalStorage() {
  const user = SessionStorage.getItem<FG.User>('currentUser') || undefined;
  if (user && user.birthday && typeof user.birthday === 'string')
    user.birthday = new Date(user.birthday);
  return user;
}
const state: UserStateInterface = {
  users: [],
  roles: [],
  permissions: [],
  currentUser: loadUserFromLocalStorage(),
  currentPermissions: SessionStorage.getItem<FG.Permission[]>('currentPermissions') || [],
  loading: 0,
};

const mutations: MutationTree<UserStateInterface> = {
  setCurrentUser(state, data: FG.User) {
    if (typeof data.birthday === 'string') data.birthday = new Date(data.birthday);
    SessionStorage.set('currentUser', data);
    state.currentUser = data;
  },
  setCurrentPermissions(state, data: FG.Permission[]) {
    SessionStorage.set('currentPermissions', data);
    state.currentPermissions = data;
  },
  clearCurrentUser(state) {
    SessionStorage.remove('currentUser');
    SessionStorage.remove('currentPermissions');
    state.currentUser = undefined;
    state.currentPermissions = [];
  },
  setUsers(state, data: FG.User[]) {
    state.users = data;
  },
  setUser(state, data: FG.User) {
    const index = state.users.findIndex((x) => x.userid === data.userid);
    if (index > -1) state.users[index] = data;
    else state.users.push(data);
  },
  setRoles(state, data: FG.Role[]) {
    state.roles = data;
  },
  addRole(state, data: FG.Role) {
    state.roles.push(data);
  },
  updateRole(state, data: FG.Role) {
    const idx = state.roles.findIndex((role) => role.id === data.id);
    if (idx >= 0) {
      state.roles[idx].name = data.name;
      state.roles[idx].permissions = data.permissions;
    }
  },
  setPermissions(state, data: FG.Permission[]) {
    state.permissions = data;
  },
  setLoading(state, data = true) {
    if (data) state.loading += 1;
    else state.loading -= 1;
  },
};

const actions: ActionTree<UserStateInterface, UserSessionState> = {
  getCurrentUser({ commit, rootState }) {
    if (rootState.sessions.currentSession) {
      commit('setLoading');
      api
        .get(`/users/${rootState.sessions.currentSession.userid}`)
        .then((response: AxiosResponse<CurrentUserResponse>) => {
          commit('setCurrentUser', response.data);
          commit('setCurrentPermissions', response.data.permissions);
        })
        .catch((err) => {
          console.warn(err);
        })
        .finally(() => {
          commit('setLoading', false);
        });
    } else {
      console.debug('User not logged in, can not get current_user.');
    }
  },

  getUsers({ commit }) {
    commit('setLoading');
    api
      .get('/users')
      .then((response: AxiosResponse<FG.User[]>) => {
        response.data.forEach((user) => {
          if (user.birthday) user.birthday = new Date(user.birthday);
        });
        commit('setUsers', response.data);
      })
      .catch((err) => {
        console.warn(err);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  updateUser({ commit, state, dispatch }, data: FG.User) {
    commit('setLoading');
    api
      .put(`/users/${data.userid}`, data)
      .then(() => {
        if (state.currentUser && state.currentUser.userid === data.userid)
          void dispatch('getCurrentUser');
        else void dispatch('getUsers');
      })
      .catch((error) => {
        console.log(error);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  uploadAvatar({ commit }, payload: { user: FG.User; file: string }) {
    commit('setLoading');
    const formData = new FormData();
    formData.append('file', payload.file);
    return api
      .post(`/users/${payload.user.userid}/avatar`, formData, {
        headers: {
          'Content-Type': 'multipart/form-data',
        },
      })
      .catch((error: AxiosError) => {
        return Promise.reject(error.response);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  setUser({ commit, state, dispatch }, data: FG.User) {
    commit('setLoading');
    api
      .post('users', data)
      .then(() => {
        if (state.currentUser && state.currentUser.userid === data.userid)
          void dispatch('getCurrentUser');
        else void dispatch('getUsers');
      })
      .catch((error) => {
        console.warn(error);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  getRoles({ commit, state }, force = false) {
    if (!force && state.roles.length > 0) return;
    commit('setLoading');
    api
      .get('/roles')
      .then((response: AxiosResponse<FG.Role[]>) => {
        commit('setRoles', response.data);
      })
      .finally(() => commit('setLoading', false));
  },

  updateRole({ commit }, data: FG.Role) {
    commit('setLoading');
    api
      .put(`/roles/${data.id}`, data)
      .then(() => {
        commit('updateRole', data);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  newRole({ commit }, data: FG.Role) {
    commit('setLoading');
    return api
      .post('/roles', data)
      .then((response: AxiosResponse<FG.Role>) => {
        commit('addRole', response.data);
        return Promise.resolve(response.data);
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  deleteRole({ commit, state }, data: FG.Role) {
    commit('setLoading');
    api
      .delete(`/roles/${data.id}`)
      .then(() => {
        commit(
          'setRoles',
          state.roles.filter((value) => value.id !== data.id)
        );
      })
      .finally(() => {
        commit('setLoading', false);
      });
  },

  getPermissions({ commit, state }, force = false) {
    if (!force && state.permissions.length > 0) return;
    commit('setLoading');
    api
      .get('/roles/permissions')
      .then((response: AxiosResponse<FG.Role[]>) => {
        commit('setPermissions', response.data);
      })
      .finally(() => commit('setLoading', false));
  },
  getUser({ commit, getters }, data: { userid: string; force?: boolean }) {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    const user = <FG.User | undefined>getters['getUser'](data.userid);
    if (user === undefined || data.force === true) {
      return api.get(`/users/${data.userid}`).then((response: AxiosResponse<FG.User>) => {
        commit('setUser', response.data);
        return response.data;
      });
    } else {
      return Promise.resolve(user);
    }
  },
};

const getters: GetterTree<UserStateInterface, UserSessionState> = {
  getUser: (state) => (userid: string) => {
    const user = state.users.filter((usr) => usr.userid === userid);
    return user.length > 0 ? user[0] : undefined;
  },
  currentUser({ currentUser }) {
    return currentUser;
  },
  users({ users }) {
    return users;
  },
  loading({ loading }) {
    return loading > 0;
  },
  displayName({ currentUser }) {
    return currentUser?.display_name;
  },
  roles({ roles }): string[] {
    return roles.map((role) => role.name).flat();
  },
};

const usersStore: Module<UserStateInterface, UserSessionState> = {
  namespaced: true,
  actions,
  getters,
  mutations,
  state,
};

export default usersStore;
